# -*- coding: UTF-8 -*-
#
# Copyright (c) 2013-2023 NIBIO.
#
# This file is part of VIPSWeb 
# (see https://gitlab.nibio.no/VIPS/VIPSWeb).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#
from django import template
from django.utils.translation import gettext as _
from django.utils import translation
from django.conf import settings
from datetime import datetime
from dateutil.relativedelta import relativedelta
from django.urls import reverse
from django.utils.safestring import mark_safe

register = template.Library()

# Reads the variable local_settings.MAIN_MENU
# Parses it, returns HTML
@register.simple_tag(takes_context=True)
def generate_main_menu(context):
	# Get the request object
	try:
		request = context['request']
	except KeyError:
		request = context['view'].request
	menu_html = ''
	# Iterating the horizontal elements
	for menu_item in settings.MAIN_MENU:
		# If URL given, it's a main menu link
		if menu_item.get("url",None) != None:
			# Adding the active class attribute if url matches request path
			active_class_attr = ' class="currentLink"' if request.path.startswith(menu_item["url"]) else ''
			menu_html += '<li><a href="%s"%s>%s</a></li>' % (menu_item["url"], active_class_attr, _(menu_item["label"]))
		# Else, it's a dropdown menu
		else:
			dropdown_html = ""
			child_item_is_active = False
			# Iterating through the vertical elements
			for menu_child_item in menu_item["child_items"]:
				if menu_child_item.get("separator",None) != None:
					dropdown_html += '<li role="separator" class="divider"></li>'
				elif menu_child_item.get("heading",None) != None:
					dropdown_html += '<li class="dropdown-header">%s</li>' % menu_child_item["heading"]
				else:
					# If url of this page matches request.path, add the active class
					# AND flag that this dropdown menu contains the active item
					#active_class_attr = ' class="currentLink"' if request.path.startswith(menu_child_item["url"]) else ''
					if request.path.startswith(menu_child_item.get("url","FooBar")):
						child_item_is_active = True
					target_attribute = ' target="%s"' % menu_child_item.get("target","") if menu_child_item.get("target",None) != None else ''
					dropdown_html += '<li><a href="%s"%s>%s</a></li>' % (menu_child_item["url"], target_attribute,_(menu_child_item["label"]))
			# If this dropdown menu contains the active item:
			# add an active class to the dropdown item
			active_class_attr = ' currentLink' if child_item_is_active else ''
			# Prepending the list item
			dropdown_html = """<li class="dropdown">
                <a href="#" class="dropdown-toggle%s" data-toggle="dropdown" role="button" aria-expanded="false">%s <span class="caret"></span></a>
                <ul class="dropdown-menu" role="menu">""" % (active_class_attr, _(menu_item["label"])) + dropdown_html
			# Wrap up the HTML
			dropdown_html += "</ul></li>"
			menu_html += dropdown_html
	# If app information is installed, generate dropdown for it
	information_installed = False
	for app_name in settings.INSTALLED_APPS:
		if app_name == "information":
			information_installed = True
	if information_installed:
		dropdown_template = """<li class="dropdown">
                <a href="#" class="dropdown-toggle%s" data-toggle="dropdown" role="button" aria-expanded="false">%s <span class="caret"></span></a>
                <ul class="dropdown-menu" role="menu">%s</ul>
                </li>"""
		item_template = """<li><a href="%s">%s</a></li>"""
		from information.models import Information, InformationLocale
		top_level = Information.objects.filter(parent_information__isnull=True)
		items_html = []
		for info in top_level:
			info_locale = InformationLocale.get_information_locale_with_fallback(info.pk, request.LANGUAGE_CODE)
			items_html.append( item_template % (reverse('information:detail', kwargs={"information_id":info.pk}), info_locale.headline) )
		menu_html += dropdown_template % (" currentLink" if request.path.startswith("/information") else "",_("Information"),"".join(items_html))
			
	return mark_safe(menu_html)

# Reads the variable local_settings.FRONTPAGE_MENU
# Parses it, returns HTML
@register.simple_tag(takes_context=True)
def generate_frontpage_menu(context):

	
	if settings.FRONTPAGE_MENU == None or len(settings.FRONTPAGE_MENU) == 0:
		return '<div class="col-md-12">&nbsp;</div>'
	
	if 12 % len(settings.FRONTPAGE_MENU) != 0:
		return """
		<div class="alert alert-danger">
  		ERROR IN MENU CONFIGURATION. 12 MUST BE DIVISABLE BY NUMBER OF COLUMNS
		</div>
		""" 
	
	menu_html = ''
	col_md = int(12/len(settings.FRONTPAGE_MENU))
	for col in settings.FRONTPAGE_MENU:
		menu_html += '<div class="col-md-%s"><h3>%s</h3>' % (col_md,_(col["label"]))
		menu_html += "<ul class='linkList'>"
		for item in col["items"]:
			menu_html += '<li><a href="%s">%s</a></li>' % (item["url"], item["label"])
		menu_html += "</ul>"
		menu_html += "</div>"
	return mark_safe(menu_html)

# Get the footer text in correct language
# REPLACED WITH THE MORE FLEXIBLE get_text_i18(text_dict) BELOW
"""
@register.simple_tag()
def get_footer_text_i18n():
	# Absolute fallback is the empty string
	footer_html = None
	# Languages in preferred order: user's selected language, default language of site, English
	language_candidates = [translation.get_language(), settings.LANGUAGE_CODE, "en"]
	for language_candidate in language_candidates:
		if settings.FOOTER_HTML.get(language_candidate, None) != None:
			footer_html = settings.FOOTER_HTML.get(language_candidate, footer_html)
			break
	# Fallback: First item in list
	if footer_html == None and len(settings.FOOTER_HTML.keys()) > 0:
		footer_html = settings.FOOTER_HTML[settings.FOOTER_HTML.keys()[0]]
	
	return footer_html if footer_html != None else ""
"""

# Get text in correct language
# Assuming text is on format{"LANGCODE":"CONTENTS","LANGCODE":"CONTENTS"}
@register.simple_tag
def get_text_i18n(text_dict):
	# If no dictionary, return as-is
	if not isinstance(text_dict,dict):
		return text_dict
	text = None
	# Languages in preferred order: user's selected language, default language of site, English
	language_candidates = [translation.get_language(), settings.LANGUAGE_CODE, "en"]
	for language_candidate in language_candidates:
		if text_dict.get(language_candidate, None) != None:
			text = text_dict.get(language_candidate, text_dict)
			break
	# Fallback: First item in list
	if text == None and len(text_dict.keys()) > 0:
		text = text_dict[text_dict.keys()[0]]
	
	return mark_safe(text) if text != None else ""

@register.simple_tag
def get_system_time(format_string):

	if(hasattr(settings, "SYSTEM_TIME_EXACT") and settings.SYSTEM_TIME_EXACT != None and settings.SYSTEM_TIME_EXACT.strip() != ""):
		time = datetime.strptime(settings.SYSTEM_TIME_EXACT,"%Y-%m-%d")
	else:
		time = datetime.now() + relativedelta(months = settings.SYSTEM_TIME_OFFSET_MONTHS)
	return time.strftime(format_string)

@register.filter
def get_range( value ):
	"""
	  Filter - returns a list containing range made from given value
	  Usage (in template):
	
	  <ul>{% for i in 3|get_range %}
	    <li>{{ i }}. Do something</li>
	  {% endfor %}</ul>
	
	  Results with the HTML:
	  <ul>
	    <li>0. Do something</li>
	    <li>1. Do something</li>
	    <li>2. Do something</li>
	  </ul>
	
	  Instead of 3 one may use the variable set in the views
	"""
	return range( value )
	